@@ -636,9 +636,9 @@ breakPointPrint(breakPointPtr breakPtr)
 
     breakStatus = breakStatusText[breakPtr->flags & BREAKPOINT_ENABLED];
     if (breakPtr->url)
-	xsldbgGenericErrorFunc(i18n("Breakpoint %1 %2 for template: \"%3\" mode: \"%4\" in file %5 at line %6").arg(breakPtr->id).arg(i18n((const char*)breakStatus)).arg(QString::fromUtf8((const char*)breakTemplate)).arg(QString::fromUtf8((const char*)breakMode)).arg(xsldbgUrl((const char*)breakPtr->url)).arg(breakPtr->lineNo));
+	xsldbgGenericErrorFunc(i18n("Breakpoint %1 %2 for template: \"%3\" mode: \"%4\" in file \"%5\" at line %6").arg(breakPtr->id).arg(i18n(breakStatus)).arg(xsldbgText(breakTemplate)).arg(xsldbgText(breakMode)).arg(xsldbgUrl(breakPtr->url)).arg(breakPtr->lineNo));
     else
-	xsldbgGenericErrorFunc(i18n("Breakpoint %1 %2 for template: \"%3\" mode: \"%4\"").arg(breakPtr->id).arg(i18n((const char*)breakStatus)).arg(QString::fromUtf8((const char*)breakTemplate)).arg(QString::fromUtf8((const char*)breakMode)));
+	xsldbgGenericErrorFunc(i18n("Breakpoint %1 %2 for template: \"%3\" mode: \"%4\"").arg(breakPtr->id).arg(i18n(breakStatus)).arg(xsldbgText(breakTemplate)).arg(xsldbgText(breakMode)));
     return ++result;
 }
 
