@@ -40,6 +40,7 @@
 #include <libxml/valid.h>
 
 //own includes
+#include "dtepeditdlg.h"
 #include "dtdparser.h"
 #include "qtag.h"
 #include "dtepcreationdlg.h"
@@ -52,9 +53,11 @@ namespace DTD
 {
   QString dirName;
   xmlDtdPtr dtd_ptr; /* Pointer to the parsed DTD */
+  QTextStream entityStream;
 }
 
 void saveElement(xmlElementPtr elem, xmlBufferPtr buf);
+void saveEntity(xmlEntityPtr entity, xmlBufferPtr buf);
 
 DTDParser::DTDParser(const KURL& dtdURL, const QString &dtepDir)
 {
@@ -77,9 +80,28 @@ bool DTDParser::parse()
   DTD::dtd_ptr = xmlParseDTD(NULL, xmlCharStrndup(fileName.utf8(), fileName.utf8().length()));
   if( DTD::dtd_ptr == NULL )
   {
-     KMessageBox::error(0, i18n("Error while parsing the DTD.\nThe error message is:\n%1").arg("x"));
-     return false;
-//          xmlGenericError(xmlGenericErrorContext,                          "Could not parse %s\n", argv[1]);
+    QString errorStr = i18n("Unknown");
+#ifndef LIBXML_2_5
+    xmlErrorPtr errorPtr = xmlGetLastError();
+    if (errorPtr != NULL)
+    {
+      QString s = QString::fromLatin1(errorPtr->message);
+      if (!s.isEmpty())
+        errorStr = s;
+      s = QString::fromLatin1(errorPtr->str1);
+      if (!s.isEmpty())
+        errorStr += "<br>" + s;
+      s = QString::fromLatin1(errorPtr->str2);
+      if (!s.isEmpty())
+        errorStr += "<br>" + s;
+      s = QString::fromLatin1(errorPtr->str2);
+      if (!s.isEmpty())
+        errorStr += "<br>" + s;
+      xmlResetError(errorPtr);
+    }
+#endif    
+    KMessageBox::error(0, i18n("<qt>Error while parsing the DTD.<br>The error message is:<br><i>%1</i></qt>").arg(errorStr));
+    return false;
   }
   KDialogBase dlg(0L, 0L, true, i18n("DTD - > DTEP Conversion"), KDialogBase::Ok | KDialogBase::Cancel);
   DTEPCreationDlg w(&dlg);
@@ -118,9 +140,34 @@ bool DTDParser::parse()
      KMessageBox::error(0, i18n("No elements were found in the DTD."));
      return false;
   }
+  if (DTD::dtd_ptr->entities)
+  {
+    QFile file( DTD::dirName + "entities.tag" );
+    if ( file.open( IO_WriteOnly ) )
+    {
+      DTD::entityStream.setDevice(&file);
+      DTD::entityStream.setEncoding(QTextStream::UnicodeUTF8);
+      DTD::entityStream << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" << endl;
+      DTD::entityStream << "<!DOCTYPE TAGS>" << endl
+          << "<TAGS>" << endl;
+      xmlHashScan((xmlEntitiesTablePtr)DTD::dtd_ptr->entities, (xmlHashScanner)saveEntity, 0);
+      DTD::entityStream << "</TAGS>" << endl;
+      file.close();
+    }
+  }  
   xmlFreeDtd(DTD::dtd_ptr);
   writeDescriptionRC();
-
+  if (w.fineTune->isChecked())
+  {
+    KDialogBase editDlg(0L, "edit_dtep", true, i18n("Configure DTEP"), KDialogBase::Ok | KDialogBase::Cancel);
+    DTEPEditDlg dtepDlg(DTD::dirName + "description.rc", &editDlg);
+    editDlg.setMainWidget(&dtepDlg);
+    if (editDlg.exec())
+    {
+      dtepDlg.saveResult();
+    }
+    
+  }
   return true;
 }
 
@@ -280,6 +327,16 @@ void saveElement(xmlElementPtr elem, xmlBufferPtr buf)
  }
 }
 
+void saveEntity(xmlEntityPtr entity, xmlBufferPtr buf)
+{
+  Q_UNUSED(buf);
+  if (entity)
+  {
+    QString name = QString((const char*)entity->name);
+    DTD::entityStream << "<tag name=\"" << name << "\" type=\"entity\" />" << endl << endl;
+  }
+}
+
 QString DTDParser::dirName()
 {
   return DTD::dirName;
