@@ -40,7 +40,6 @@
 #include "resource.h"
 #include "dtdparser.h"
 #include "document.h"
-#include "quanta.h"
 #include "viewmanager.h"
 
 #include "dtds.h"
@@ -52,9 +51,12 @@ const QString m_rcFilename("description.rc");
   This constructor reads the dictionary of known dtd's, the attributes and tags will be loaded
   on the first access to one dtd.
 */
-DTDs::DTDs()
+DTDs::DTDs(QObject *parent)
+  :QObject(parent)
 {
-  connect(this, SIGNAL(hideSplash()), quantaApp, SLOT(slotHideSplash()));
+  connect(this, SIGNAL(hideSplash()), parent, SLOT(slotHideSplash()));
+  connect(this, SIGNAL(enableIdleTimer(bool)), parent, SLOT(slotEnableIdleTimer(bool)));
+  connect(this, SIGNAL(loadToolbarForDTD(const QString&)), parent, SLOT(slotLoadToolbarForDTD(const QString&)));
 //  kdDebug(24000) << "dtds::dtds" << endl;
   m_dict = new QDict<DTDStruct>(119, false); //optimized for max 119 DTD. This should be enough.
   m_dict->setAutoDelete(true);
@@ -133,6 +135,8 @@ bool DTDs::readTagDir(const QString &dirName, bool loadAll)
   dtd->name      = dtdName;
   dtd->nickName  = dtdConfig->readEntry("NickName", dtdName);
   dtd->mimeTypes = dtdConfig->readListEntry("MimeTypes");
+  for (uint i = 0; i < dtd->mimeTypes.count(); i++)
+    dtd->mimeTypes[i] = dtd->mimeTypes[i].stripWhiteSpace();
   dtd->family    = dtdConfig->readNumEntry("Family", Xml);
   if (dtd->family != Xml)
       dtd->toplevel = dtdConfig->readBoolEntry("TopLevel", false);
@@ -183,7 +187,6 @@ bool DTDs::readTagDir2(DTDStruct *dtd)
   if (!QFile::exists(dtd->fileName)) return false;
 
   kapp->setOverrideCursor( QCursor(Qt::WaitCursor) );
-//  quantaApp->slotStatusMsg(i18n("Loading DTD's..."));
 
   KConfig *dtdConfig = new KConfig(dtd->fileName, true);
 
@@ -215,9 +218,11 @@ bool DTDs::readTagDir2(DTDStruct *dtd)
   QString dirName = dirURL.path(1);
   if (QFile::exists(dirName + "common.tag"))
     readTagFile(dirName + "common.tag", dtd, 0L);
-  bool idleTimerStatus = quantaApp->enableIdleTimer(false);
+  //bool idleTimerStatus = quantaApp->slotEnableIdleTimer(false);
+  emit enableIdleTimer(false);
   KURL::List files = QExtFileInfo::allFilesRelative(dirURL, "*.tag");
-  quantaApp->enableIdleTimer(idleTimerStatus);
+  //quantaApp->slotEnableIdleTimer(idleTimerStatus);
+  emit enableIdleTimer(true);
   QString tmpStr;
   for ( KURL::List::Iterator it_f = files.begin(); it_f != files.end(); ++it_f )
   {
@@ -336,8 +341,9 @@ bool DTDs::readTagDir2(DTDStruct *dtd)
       dtd->specialAreaNames["<?xml"] = "XML PI";
       dtd->specialAreas["<!--"] = "-->";
       dtd->specialAreaNames["<!--"] = "comment";
-      dtd->specialAreas["<!"] = ">";
-      dtd->specialAreaNames["<!"] = "DTD";
+//      dtd->specialAreas["<!"] = ">";
+  //    dtd->specialAreaNames["<!"] = "DTD";
+      dtd->insideDTDs.append("dtd");
       tmpStr = "(<?xml)|(<!--)|(<!)|";
       rxStr = QuantaCommon::makeRxCompatible(tmpStr);
   }
@@ -393,7 +399,7 @@ bool DTDs::readTagDir2(DTDStruct *dtd)
   /**** End of code for the new parser *****/
 
   //read the definition of a structure, and the structure keywords
-  QStringList structKeywords = dtdConfig->readListEntry("StructKeywords",';');
+  QStringList structKeywords = dtdConfig->readListEntry("StructKeywords",',');
   if (structKeywords.count() !=0 )
   {
       tmpStr = "\\b(";
@@ -409,7 +415,7 @@ bool DTDs::readTagDir2(DTDStruct *dtd)
   }
   dtd->structKeywordsRx.setPattern(tmpStr);
 
-  structKeywords = dtdConfig->readListEntry("LocalScopeKeywords",';');
+  structKeywords = dtdConfig->readListEntry("LocalScopeKeywords",',');
   if (structKeywords.count() !=0 )
   {
       tmpStr = "\\b(";
@@ -454,6 +460,15 @@ bool DTDs::readTagDir2(DTDStruct *dtd)
   dtd->booleanTrue = dtdConfig->readEntry("BooleanTrue","true");
   dtd->booleanFalse = dtdConfig->readEntry("BooleanFalse","false");
   dtd->singleTagStyle = dtdConfig->readEntry("Single Tag Style", "xml").lower();
+  dtd->variableGroupIndex = dtdConfig->readNumEntry("VariableGroupIndex", 0) - 1;
+  dtd->functionGroupIndex = dtdConfig->readNumEntry("FunctionGroupIndex", 0) - 1;
+  dtd->classGroupIndex = dtdConfig->readNumEntry("ClassGroupIndex", 0) - 1;
+  if (dtd->classGroupIndex != -1)
+  {
+    tmpStr = dtdConfig->readEntry("MemberAutoCompleteAfter").stripWhiteSpace();
+    dtd->memberAutoCompleteAfter.setPattern(tmpStr);
+  }
+  dtd->objectGroupIndex = dtdConfig->readNumEntry("ObjectGroupIndex", 0) - 1;
 
   //read the definition of different structure groups, like links, images, functions
   //classes, etc.
@@ -473,23 +488,16 @@ bool DTDs::readTagDir2(DTDStruct *dtd)
         group.name = dtdConfig->readEntry("Name").stripWhiteSpace();
         group.noName = dtdConfig->readEntry("No_Name").stripWhiteSpace();
         group.icon = dtdConfig->readEntry("Icon").stripWhiteSpace();
-        tmpStr = dtdConfig->readEntry("SearchRx").stripWhiteSpace();
-        group.searchRx.setPattern(tmpStr);
-        group.hasSearchRx = !group.searchRx.pattern().isEmpty();
-        group.isMinimalSearchRx = dtdConfig->readBoolEntry("SearchRx_Minimal", false);
-        group.clearRx = dtdConfig->readEntry("ClearRx").stripWhiteSpace();
-        tagStr = dtdConfig->readEntry("Tag").stripWhiteSpace();
-        group.tag = "";
-        if (!tagStr.isEmpty())
-        {
-          attrRx.search(tagStr);
-          tmpStr = attrRx.cap();
-          tmpStrList = QStringList::split(',', tmpStr.mid(1, tmpStr.length()-2));
-          group.tag = tagStr.left(tagStr.find('(')).lower();
-          group.attributes.clear();
-          for (uint i = 0; i < tmpStrList.count(); i++)
-            group.attributes += tmpStrList[i].stripWhiteSpace();
-        }
+        tmpStr = dtdConfig->readEntry("DefinitionRx").stripWhiteSpace();
+        group.definitionRx.setPattern(tmpStr);
+        tmpStr = dtdConfig->readEntry("UsageRx").stripWhiteSpace();
+        group.usageRx.setPattern(tmpStr);
+        tmpStr = dtdConfig->readEntry("TypeRx").stripWhiteSpace();
+        group.typeRx.setPattern(tmpStr);
+        group.hasDefinitionRx = !group.definitionRx.pattern().isEmpty();
+        group.isMinimalDefinitionRx = dtdConfig->readBoolEntry("DefinitionRx_Minimal", false);
+        group.appendToTags = dtdConfig->readBoolEntry("AppendToTags", false);
+        group.parentGroup = dtdConfig->readEntry("ParentGroup").stripWhiteSpace();
         tagStr = dtdConfig->readEntry("TagType", "Text").stripWhiteSpace();
         if (tagStr == "XmlTag")
             group.tagType = Tag::XmlTag;
@@ -529,6 +537,8 @@ bool DTDs::readTagDir2(DTDStruct *dtd)
         group.name = dtdConfig->readEntry("Name").stripWhiteSpace();
         group.noName = dtdConfig->readEntry("No_Name").stripWhiteSpace();
         group.icon = dtdConfig->readEntry("Icon").stripWhiteSpace();
+        group.appendToTags = dtdConfig->readBoolEntry("AppendToTags", false);
+        group.parentGroup = dtdConfig->readEntry("ParentGroup").stripWhiteSpace();
         QString tagStr = dtdConfig->readEntry("Tag").stripWhiteSpace();
         if (!tagStr.isEmpty())
         {
@@ -546,6 +556,7 @@ bool DTDs::readTagDir2(DTDStruct *dtd)
         }
       }
     }
+
   delete dtdConfig;
   dtd->loaded = true;
   resolveInherited(dtd);
@@ -635,15 +646,24 @@ uint DTDs::readTagFile(const QString& fileName, DTDStruct* parentDTD, QTagList *
  for (uint i = 0; i < numOfTags; i++)
  {
     QDomNode n = nodeList.item(i);
+    QDomElement e = n.toElement();
+    if (e.attribute("type") == "class")
+    {
+      QString extends = e.attribute("extends");
+      QString name = e.attribute("name");
+      if (!name.isEmpty() && !extends.isEmpty())
+        parentDTD->classInheritance[name] = extends;
+      continue;
+    }
     QTag *tag = new QTag();
-    tag->setName(n.toElement().attribute("name"));
+    tag->setName(e.attribute("name"));
     tag->setFileName(fileName);
     tag->parentDTD = parentDTD;
     bool common = false;
     setAttributes(&n, tag, common);
     if (common)
     {
-      QString groupName = n.toElement().attribute("name");
+      QString groupName = e.attribute("name");
       AttributeList *attrs = tag->attributes();
       attrs->setAutoDelete(false);
       AttributeList *commonAttrList = new AttributeList;      //no need to delete it
@@ -703,12 +723,18 @@ void DTDs::setAttributes(QDomNode *dom, QTag* tag, bool &common)
       tag->setOptional(true);
     }
 
-    tag->type = el.attribute("type","xmltag");
-    tag->returnType = el.attribute("returnType","");
+    tag->type = el.attribute("type", "xmltag");
+    tag->returnType = el.attribute("returnType", "");
+    tag->className = el.attribute("class", "");
+    tag->comment = el.attribute("comment", "");
+    if (!tag->comment.isEmpty())
+      tag->comment = " [" + i18n(tag->comment) + "] ";
+    tag->comment.prepend(el.attribute("version"));
  } else
  {
    common = true;
  }
+ QString attrList;
  for ( QDomNode n = dom->firstChild(); !n.isNull(); n = n.nextSibling() )
  {
    tmpStr = n.nodeName();
@@ -790,10 +816,21 @@ void DTDs::setAttributes(QDomNode *dom, QTag* tag, bool &common)
        attr->values.append("Teal");
        attr->values.append("Aqua");
      } else if ( attr->type == "url" ) {
+       //not treated yet
      } else if ( attr->type == "input" ) {
-     } else {
+       //not treated yet
      }
 
+     if (tag->type == "function" || tag->type == "method")
+     {
+      if (attr->status == "optional")
+      {
+        attrList = attrList + "["+attr->type +" "+attr->name +"], ";
+      } else
+      {
+        attrList = attrList + attr->type +" "+attr->name +", ";
+      }
+     }
      if (!attr->name.isEmpty())
      {
        tag->addAttribute(attr);
@@ -801,7 +838,8 @@ void DTDs::setAttributes(QDomNode *dom, QTag* tag, bool &common)
      delete attr;
    }
  }
-
+ if (!attrList.isEmpty())
+   tag->comment.prepend(attrList.left(attrList.length() - 2) + "; ");
 }
 
 
@@ -833,7 +871,7 @@ void DTDs::slotLoadDTD()
               KMessageBox::questionYesNo(0L, i18n("<qt>Use the newly loaded <b>%1</b> DTD for the current document?</qt>").arg(nickName), i18n("Change DTD")) == KMessageBox::Yes)
           {
             w->setDTDIdentifier(dtdName);
-            quantaApp->loadToolbarForDTD(w->getDTDIdentifier()); //fixme: make signal no direct call
+            emit loadToolbarForDTD(w->getDTDIdentifier());
             emit forceReparse();
           }
       }
@@ -859,7 +897,7 @@ void DTDs::slotLoadDTEP(const QString &_dirName, bool askForAutoload)
   removeDTD(dtd);
   if (!readTagDir(dirName))
   {
-    KMessageBox::error(0L, i18n("<qt>Cannot read the DTEP from <b>%1</b>. Check that the directory contains a valid DTEP (<i>description.rc and *.tag files</i>).</qt>").arg(dirName), i18n("Error loading DTEP"));
+    KMessageBox::error(0L, i18n("<qt>Cannot read the DTEP from <b>%1</b>. Check that the folder contains a valid DTEP (<i>description.rc and *.tag files</i>).</qt>").arg(dirName), i18n("Error Loading DTEP"));
   } else
   {
     QString family = dtdcfg.readEntry("Family", "1");
@@ -877,7 +915,7 @@ void DTDs::slotLoadDTEP(const QString &_dirName, bool askForAutoload)
         KMessageBox::questionYesNo(0L, i18n("<qt>Use the newly loaded <b>%1</b> DTD for the current document?</qt>").arg(nickName), i18n("Change DTD")) == KMessageBox::Yes)
     {
       w->setDTDIdentifier(dtdName);
-      quantaApp->loadToolbarForDTD(w->getDTDIdentifier()); //fixme: make signal no direct call
+      emit loadToolbarForDTD(w->getDTDIdentifier());
       emit forceReparse();
     }
   }
